/**
     * Invariant (contrat) d'état interne:
     *     <pre>
     *             this.getPlafond() > 0      (plafond toujours positif)
     *             &&   this.getSolde() >= 0  (solde supérieur ou égal à zéro)
     *             &&   this.getSolde() <= this.getPlafond() (solde inférieur ou égal au plafond)
     *             &&   (!(this.estOuvert())  alors this.getSolde() = 0 ) (un compte fermé doit avoir solde=0)
     *     </pre>
     * @author aponte
     */
package compteLivretA;


public class CompteLivretA {
    private double solde;
    private double plafondMax;
    private boolean compteOuvert = false;
    private String nomTitulaire;
    
    // A tester
    /**
     * Création d'un compte avec état ouvert.
     * @param nom nom du titulaire
     * @param soldeinit solde initial
     * @param plafondm plafond maximal autorisé par la loi.
     * @throws IllegalArgumentException 
     *    si {@code soldeinit < 0 ||  plafondm <= 0}
     * @throws IllegalStateException 
     *    si {@code soldeinit > plafondm } 
     */
    public CompteLivretA(String nom, double soldeinit, double plafondm){
        if (soldeinit < 0 || plafondm <= 0) 
            throw new IllegalArgumentException();
        if (soldeinit > plafondm )
            throw new IllegalStateException("Le solde initial dépasse le plafond autorisé.");
        solde = soldeinit; 
        plafondMax = plafondm;  
        compteOuvert = true;  
        nomTitulaire=nom;
    }
    // Utilisé pour les tests uniquement
    // A tester.
    /** 
     * Création d'un compte avec solde 0 et état fermé, à utiliser uniquement pour les tests.
     * @param nom nom du titulaire
     * @param plafondm plafond maximal autorisé par la loi.
     * @throws IllegalArgumentException si {@code plafondm <= 0}
     */
    public CompteLivretA(String nom, double plafondm){
        if ( plafondm <= 0)
            throw new IllegalArgumentException();
        plafondMax = plafondm;  
        compteOuvert = false; 
        solde = 0;
        nomTitulaire=nom;
    }  
    
   /**
    * Retourne le nom du titulaire du compte.
    * @return nom du titulaire du compte
    */
    public String getNomTitulaire(){
    return nomTitulaire;
}
    /**
     * Retourne la valeur du plafond maximal autorisé par la loi.
     * @return valeur du plafond maximal autorisé par la loi
     */
    public double getPlafond(){
        return this.plafondMax;
    }
    
    /**
     * Retourne le solde courant du compte.
     * @return solde du compte
     */
    public double getSolde(){
        return solde;
    }
    
    /**
     * Retourne true si le compte est ouvert, false sinon.
     * @return true si le compte est ouvert, false sinon.
     */
    public boolean estOuvert(){
        return this.compteOuvert;
    }
     // A tester
    /**
     * Ajoute m au solde courant si cet ajout ne fait pas dépaser le plafond maximal autorisé.
     * @param m montant du dépôt
     * @throws IllegalArgumentException si {@code m < 0}
     * @throws OperationInterdite si {@code m+this.getSolde() > this.getPlafond()}
     * @throws OperationInterdite si le compte n'est pas ouvert. 
     */
    public void depot(double m){
        if (m<0 )
              throw new IllegalArgumentException();
        if (!estOuvert())
            throw new OperationInterdite("Dépot impossible sur un compte fermé.");      
        if (m + this.getSolde() > this.getPlafond())
            throw new OperationInterdite("Dépot avec dépassement du plafond.");      
        solde = solde + m;
    }
    
    // A tester
    /**
     * Soustrait m au solde courant du compte si celui-ci à un solde supérieur ou égal à m.
     * @param m 
     * @throws IllegalArgumentException si {@code m < 0} 
     * @throws OperationInterdite si {@code this.getSolde() < m}
     * @throws OperationInterdite si le compte n'est pas ouvert. 
     */
     public void retrait(double m){
        if (!estOuvert())
            throw new OperationInterdite("Retrait interdit sur un compte fermé.");
        if (m > this.getSolde())
              throw new OperationInterdite("Impossible de faire un retrait qui dépasse le solde.");;
        if (m<0 || m > this.getSolde())
              throw new IllegalArgumentException();
        solde = solde - m;        
     }
    
    // A tester
    /** 
     * Fait passer un compte dans l'état ouvert.
     */
    public void ouvrir(){ 
       compteOuvert = true;
    }
    // Contrat javadoc à écrire
    // A tester ensuite  
    /** 
     * Fait passer un compte à l'état fermé si son solde est à zéro.
     * @throws OperationInterdite si le compte est ouvert mais pas à zéro.
     */
     public void fermer(){
        if (estOuvert() && this.getSolde() != 0)
             throw new OperationInterdite("Impossible de fermer un compte créditeur.");
        if (this.getSolde() == 0)
            compteOuvert = false;     
    } 
  @Override
    public String toString(){
        return "nom= "+ nomTitulaire+" solde= "+ solde+" ouvert= "+ compteOuvert+" plafond = "+ plafondMax;
    }
}
